/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "edge.h"

// + represents selection: Select the proper edge constraints
// (a constraint that may choose two different paths)
const edge& edge::operator+=(const edge& e)
{
    min = std::min(min, e.min);
    max = std::max(max, e.max);
    if(state == UNKNOWN or e.state == UNKNOWN)
        state = UNKNOWN;
    else if(state == NULL and e.state == NULL)
        state = NULL;
    else
        state = NORMAL;
    return *this;
}

// * represents addition (a constraint over several consecutive edges)
const edge& edge::operator*=(const edge& e)
{
    if(state == NULL or e.state == NULL)
        *this = edge();
    if(state == UNKNOWN or e.state == UNKNOWN)
        *this = unknown();
    else
    {
        if(min == rt_inf or e.min == rt_inf)
            min = rt_inf;
        else
            min += e.min;
        if(max == rt_inf or e.max == rt_inf)
            max = rt_inf;
        else
            max += e.max;
    }
    return *this;
}

std::ostream& operator<<(std::ostream& os, const edge& e)
{
    if(e.state == edge::NULL)
        os << "       ( null )       ";
    else if(e.state == edge::UNKNOWN)
        os << "      ( unknown )     ";
    else if(e.min == rt_inf and e.max == rt_zero)
        os << "    ( impossible )    ";
    else
    {
        os << "(";
        if(e.min == rt_inf)
            os << " infinity";
        else
            os << e.min;
        os << ", ";
        if(e.max == rt_inf)
            os << " infinity)";
        else
            os << e.max << ")";
    }
    return os;
}
