%{
/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../visitors/ast.h"

extern "C"
{
#include <stdlib.h>
#include <string.h>
#include "parseunion.h"

    int lineno = 1;
    struct node;
    extern node* root;
    int yyparse(void);
    int yylex(void);
    int yywrap() { return 1; }
    int yyerror(const char* msg);
    
}
%}


%token <txt>      PASS ID INT_LIT FLOAT_LIT STRING_LIT

%token <sym>      ERROR;

%token <sym>      PAREN_OPEN PAREN_CLOSE BRACKET_OPEN BRACKET_CLOSE BRACE_OPEN BRACE_CLOSE

%token <sym>      PLUS MINUS TIMES DIVIDE LESS LESS_EQ GREATER GREATER_EQ EQUALS NEQUALS ASSIGN COMMA
%token <sym>      UPLUS UMINUS
%token <sym>      DOT ARROW LINK

%token <sym>      ELSE IF INT FLOAT VOID WHILE STRING RETURN

%token <sym>      SEMICOLON
%token <sym>      ACTOR GLOBAL SYNCHRONIZER TRIGGER ACTION CONSTRAINT MIN MAX ENABLE DISABLE INIT WHEN SYNC EXCEPTION


%left PLUS MINUS
%left TIMES DIVIDE
%left UPLUS UMINUS


%type <program_node_t> program
%type <decl_list_node_t> declaration_list
%type <var_decl_node_t> var_declaration
%type <type_spec_node_t> type_specifier
%type <fun_decl_node_t> fun_declaration
%type <params_node_t> params
%type <param_list_node_t> param_list
%type <param_node_t> param
%type <compound_stmt_node_t> compound_stmt
%type <loc_decl_node_t> local_declarations
%type <stmt_list_node_t> statement_list
%type <stmt_node_t> statement
%type <expr_stmt_node_t> expression_stmt
%type <select_stmt_node_t> selection_stmt
%type <iter_stmt_node_t> iteration_stmt
%type <return_stmt_node_t> return_stmt
%type <expr_node_t> expression
%type <var_node_t> var
%type <simple_expr_node_t> simple_expression
%type <relop_node_t> relop
%type <add_expr_node_t> additive_expression
%type <addop_node_t> addop
%type <term_node_t> term
%type <mulop_node_t> mulop
%type <factor_node_t> factor
%type <call_node_t> call
%type <args_node_t> args
%type <arg_list_node_t> arg_list
%type <actor_node_t> actor_block
%type <sync_node_t> sync_block
%type <global_sync_t> global_sync
%type <init_node_t> init
%type <sync_init_node_t> sync_init
%type <sync_decl_list_node_t> sync_decl_list
%type <constraint_block_node_t> constraint_block
%type <constraint_list_node_t> constraint_list
%type <constraint_node_t> constraint
%type <trigger_block_node_t> trigger_block
%type <trigger_list_node_t> trigger_list
%type <trigger_node_t> trigger
%type <actor_params_node_t> actor_params
%type <actor_param_node_t> actor_param
%type <var_decls_node_t> var_decls
%type <func_decls_node_t> func_decls
%type <module_list_node_t> module_list
%type <module_node_t> module
%type <call_stmt_node_t> call_stmt
%type <sync_call_stmt_node_t> sync_call_stmt
%type <exception_block_node_t> exception_block

%start program


%%


program             :    module_list
                         {
                             root = new program_node(lineno, $1);
                             $$ = (program_node*)root;
                         }
                    ;
module_list         :    module_list module
                         { $$ = new module_list_node(lineno, $1, $2); }
                    |    module
                         { $$ = new module_list_node(lineno, 0, $1); }
                    ;
module              :    actor_block
                         { $$ = $1; }
                    |    sync_block
                         { $$ = $1; }
                    |    global_sync
                         { $$ = $1; }
                    |    PASS
                         { $$ = new pass_node(lineno, $1); }
                    ;
actor_block         :   ACTOR ID BRACE_OPEN declaration_list BRACE_CLOSE
                        { $$ = new actor_node(lineno, $2, $4); }
                    ;
global_sync         :   GLOBAL BRACE_OPEN sync_decl_list BRACE_CLOSE
                        { $$ = new global_sync_node(lineno, $3); }
                    ;
sync_block          :   SYNCHRONIZER ID BRACE_OPEN sync_decl_list BRACE_CLOSE
                        { $$ = new sync_node(lineno, $2, $4); }
                    ;
declaration_list    :   var_decls init func_decls
                        { $$ = new decl_list_node(lineno, $1, $2, $3); }
                    ;
var_decls           :   var_decls var_declaration
                        { $$ = new var_decls_node(lineno, $1, $2); }
                    |   var_declaration
                        { $$ = new var_decls_node(lineno, 0, $1); }
                    |   
                        { $$ = 0; }
                    ;
func_decls          :   func_decls fun_declaration
                        { $$ = new func_decls_node(lineno, $1, $2); }
                    |   fun_declaration
                        { $$ = new func_decls_node(lineno, 0, $1); }
                    |
                        { $$ = 0; }
                    ;
var_declaration     :   type_specifier ID SEMICOLON
                        { $$ = new simple_var_decl_node(lineno, $1, $2); }
                    |   type_specifier ID BRACKET_OPEN INT_LIT BRACKET_CLOSE SEMICOLON
                        { $$ = new arr_var_decl_node(lineno, $1, $2, $4); }
                    ;
type_specifier      :   INT
                        { $$ = new type_spec_node(lineno, TK_INT); }
                    |   FLOAT
                        { $$ = new type_spec_node(lineno, TK_FLOAT); }
                    |   STRING
                        { $$ = new type_spec_node(lineno, TK_STRING); }
                    |   VOID
                        { $$ = new type_spec_node(lineno, TK_VOID); }
                    ;
init                :   INIT compound_stmt
                        { $$ = new init_node(lineno, $2); }
                    /*|
                        { $$ = 0; }*/
                    ;
sync_init           :   INIT PAREN_OPEN actor_params PAREN_CLOSE compound_stmt
                        { $$ = new sync_init_node(lineno, $3, $5); }
                    ;
sync_decl_list      :   var_decls sync_init constraint_block trigger_block exception_block
                        { $$ = new sync_decl_list_node(lineno, $1, $2, $3, $4, $5); }
                    ;
constraint_block    :   CONSTRAINT BRACE_OPEN constraint_list BRACE_CLOSE
                        { $$ = new constraint_block_node(lineno, $3); }
                    ;
constraint_list     :   constraint_list constraint
                        { $$ = new constraint_list_node(lineno, $1,$2); }
                    |   constraint
                        { $$ = new constraint_list_node(lineno, 0,$1); }
                    |
                        { $$ = 0; }
                    ;
constraint          :   MIN ID DOT ID ARROW ID DOT ID ASSIGN additive_expression SEMICOLON
                        { $$ = new constraint_node(lineno, 0, $2, $4, $6, $8, $10); }
                    |   MAX ID DOT ID ARROW ID DOT ID ASSIGN additive_expression SEMICOLON
                        { $$ = new constraint_node(lineno, 1, $2, $4, $6, $8, $10); }
                    ;
trigger_block       :   TRIGGER BRACE_OPEN trigger_list BRACE_CLOSE
                        { $$ = new trigger_block_node(lineno, $3); }
                    ;
trigger_list        :   trigger_list trigger
                        { $$ = new trigger_list_node(lineno, $1, $2); }
                    |   trigger
                        { $$ = new trigger_list_node(lineno, 0, $1); }
                    |
                        { $$ = 0; }
                    ;
trigger             :   ENABLE PAREN_OPEN ID DOT ID PAREN_CLOSE WHEN expression SEMICOLON
                        { $$ = new ed_trigger_node(lineno, 1, $3, $5, $8); }
                    |   DISABLE PAREN_OPEN ID DOT ID PAREN_CLOSE WHEN expression SEMICOLON
                        { $$ = new ed_trigger_node(lineno, 0, $3, $5, $8); }
                    |   ACTION PAREN_OPEN ID DOT ID PAREN_CLOSE compound_stmt
                        { $$ = new action_trigger_node(lineno, $3, $5, $7); }
                    ;
exception_block     :   EXCEPTION compound_stmt
                        { $$ = new exception_block_node(lineno, $2); }
                    ;
actor_params        :   actor_params COMMA actor_param
                        { $$ = new actor_params_node(lineno, $1, $3); }
                    |   actor_param
                        { $$ = new actor_params_node(lineno, 0, $1); }
                    ;
actor_param         :   ID LINK ID
                        { $$ = new actor_param_node(lineno, $1, $3); }
                    |   ID
                        { $$ = new actor_param_node(lineno, $1, 0); }
                    ;
fun_declaration     :   ID PAREN_OPEN params PAREN_CLOSE compound_stmt
                        { $$ = new fun_decl_node(lineno, $1, $3, $5); }
                    ;
params              :   param_list
                        { $$ = $1; }
                    |   VOID
                        { $$ = 0; }
                    |
                        { $$ = 0; }
                    ;
param_list          :   param_list COMMA param
                        { $$ = new param_list_node(lineno, $1, $3); }
                    |   param
                        { $$ = new param_list_node(lineno, 0, $1); }
                    ;
param               :   type_specifier ID
                        { $$ = new simple_param_node(lineno, $1, $2); }
                    |   type_specifier ID BRACKET_OPEN BRACKET_CLOSE
                        { $$ = new arr_param_node(lineno, $1, $2); }
                    ;
compound_stmt       :   BRACE_OPEN local_declarations statement_list BRACE_CLOSE
                        { $$ = new compound_stmt_node(lineno, $2, $3); }
                    ;
local_declarations  :   local_declarations var_declaration
                        { $$ = new loc_decl_node(lineno, $1, $2); }
                    |
                        { $$ = 0; }
                    ;
statement_list      :   statement_list statement
                        { $$ = new stmt_list_node(lineno, $1, $2); }
                    |
                        { $$ = 0; }
                    ;
statement           :   expression_stmt
                        { $$ = $1; }
                    |   compound_stmt
                        { $$ = $1; }
                    |   selection_stmt
                        { $$ = $1; }
                    |   iteration_stmt
                        { $$ = $1; }
                    |   call_stmt
                        { $$ = $1; }
                    |   sync_call_stmt
                        { $$ = $1; }
                    |   return_stmt
                        { $$ = $1; }
                    ;
expression_stmt     :   expression SEMICOLON
                        { $$ = new expr_stmt_node(lineno, $1); }
                    |   SEMICOLON
                        { $$ = 0; }
                    ;
selection_stmt      :   IF PAREN_OPEN expression PAREN_CLOSE statement
                        { $$ = new select_stmt_node(lineno, $3, $5, 0); }
                    |   IF PAREN_OPEN expression PAREN_CLOSE statement ELSE statement
                        { $$ = new select_stmt_node(lineno, $3, $5, $7); }
                    ;
iteration_stmt      :   WHILE PAREN_OPEN expression PAREN_CLOSE statement
                        { $$ = new iter_stmt_node(lineno, $3, $5); }
                    ;
return_stmt         :   RETURN expression SEMICOLON
                        { $$ = new return_stmt_node(lineno, $2); }
                    ;
call_stmt           :   ID DOT ID PAREN_OPEN args PAREN_CLOSE SEMICOLON
                        { $$ = new call_stmt_node(lineno, $1, $3, $5); }
                    ;
sync_call_stmt      :   SYNC ID DOT ID PAREN_OPEN args PAREN_CLOSE SEMICOLON
                        { $$ = new sync_call_stmt_node(lineno, $2, $4, $6); }
                    ;
expression          :   var ASSIGN expression
                        { $$ = new assign_expr_node(lineno, $1, $3); }
                    |   simple_expression
                        { $$ = $1; }
                    ;
var                 :   ID
                        { $$ = new simple_var_node(lineno, $1); }
                    |   ID BRACKET_OPEN expression BRACKET_CLOSE
                        { $$ = new arr_var_node(lineno, $1, $3); }
                    ;
simple_expression   :   additive_expression relop additive_expression
                        { $$ = new compound_simple_expr_node(lineno, $1, $2, $3); }
                    |   additive_expression
                        { $$ = $1; }
                    ;
relop               :   LESS_EQ
                        { $$ = new relop_node(lineno, REL_LESS_EQ); }
                    |   LESS
                        { $$ = new relop_node(lineno, REL_LESS); }
                    |   GREATER
                        { $$ = new relop_node(lineno, REL_GREATER); }
                    |   GREATER_EQ
                        { $$ = new relop_node(lineno, REL_GREATER_EQ); }
                    |   EQUALS
                        { $$ = new relop_node(lineno, REL_EQUALS); }
                    |   NEQUALS
                        { $$ = new relop_node(lineno, REL_NEQUALS); }
                    ;
additive_expression :   additive_expression addop term
                        { $$ = new compound_add_expr_node(lineno, $1, $2, $3); }
                    |   term
                        { $$ = $1; }
                    ;
addop               :   PLUS
                        { $$ = new addop_node(lineno, ADD_PLUS); }
                    |   MINUS
                        { $$ = new addop_node(lineno, ADD_MINUS); }
                    ;
term                :   term mulop factor
                        { $$ = new compound_term_node(lineno, $1, $2, $3); }
                    |   factor
                        { $$ = $1; }
                    ;
mulop               :   TIMES
                        { $$ = new mulop_node(lineno, MUL_TIMES); }
                    |   DIVIDE
                        { $$ = new mulop_node(lineno, MUL_DIVIDE); }
                    ;
factor              :   PAREN_OPEN expression PAREN_CLOSE
                        { $$ = new paren_factor_node(lineno, $2); }
                    |   var
                        { $$ = new var_factor_node(lineno, $1); }
                    |   INT_LIT
                        { $$ = new num_factor_node(lineno, $1, expr_node::EX_INT); }
                    |   call
                        { $$ = $1; }
                    |   FLOAT_LIT
                        { $$ = new num_factor_node(lineno, $1, expr_node::EX_FLOAT); }
                    |   STRING_LIT
                        { $$ = new num_factor_node(lineno, $1, expr_node::EX_STRING); }
                    |   MINUS factor
                        {
                            char* p = (char*) malloc(strlen($2->text)+2);
                            p[0] = '-';
                            strcpy(p+1, $2->text);
/* BAD! BAD! BAD! */
                            $$ = new num_factor_node(lineno, p, 666);
                        }
                        %prec UMINUS
                    |   PLUS factor
                        { $$ = $2; }
                        %prec UPLUS
                    ;
call                :   ID PAREN_OPEN args PAREN_CLOSE
                        { $$ = new call_node(lineno, $1, $3); }
                    ;
args                :   arg_list
                        { $$ = new args_node(lineno, $1); }
                    |
                        { $$ = 0; }
                    ;
arg_list            :   arg_list COMMA expression
                        { $$ = new arg_list_node(lineno, $1, $3); }
                    |   expression
                        { $$ = new arg_list_node(lineno, 0, $1); }
                    ;

%%

int yyerror(const char* msg)  {
    printf("yyerror on line %d: %s\n", lineno, msg);
    printf("Token %d or string \"%s\"\n", yylval.sym, yylval.txt);
    return 1;
}

#include "lex.yy.c"
