%start st_rtsync st_rtsync_com_begin st_rtsync_com_end st_com_begin st_com_end

%{

/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

int scope_ctr = 0;
//int yywrap();

%}

DIGIT       [0-9]
LETTER      [a-zA-Z]
WHITESPACE  [ \t\r]
START_COM   "/""*"

%%

<INITIAL>"\n" {
    lineno++;
    yylval.txt = strdup(yytext);
    return PASS;
}
"\n"                        { lineno++; }


<INITIAL>{START_COM}        { BEGIN st_com_begin; }
<st_com_begin>"*"           { BEGIN st_com_end; }
<st_com_end>"/"             { BEGIN 0; }
<st_com_end>[^*/]           { BEGIN st_com_begin; }

<st_rtsync>{START_COM}      { BEGIN st_rtsync_com_begin; }
<st_rtsync_com_begin>"*"    { BEGIN st_rtsync_com_end; }
<st_rtsync_com_end>"/"      { BEGIN st_rtsync; }
<st_rtsync_com_end>[^*/]    { BEGIN st_rtsync_com_begin; }

<INITIAL>"actor" {
    BEGIN st_rtsync;
    return ACTOR;
}

<INITIAL>"synchronizer" {
    BEGIN st_rtsync;
    return SYNCHRONIZER;
}

<INITIAL>"global" {
    BEGIN st_rtsync;
    return GLOBAL;
}

<st_rtsync>{WHITESPACE}+    { ; }



<st_rtsync>"else"           { return ELSE; }
<st_rtsync>"if"             { return IF; }
<st_rtsync>"int"            { return INT; }
<st_rtsync>"float"          { return FLOAT; }
<st_rtsync>"string"         { return STRING; }
<st_rtsync>"void"           { return VOID; }
<st_rtsync>"while"          { return WHILE; }
<st_rtsync>"return"         { return RETURN; }

<st_rtsync>"trigger"        { return TRIGGER; }
<st_rtsync>"constraint"     { return CONSTRAINT; }
<st_rtsync>"min"            { return MIN; }
<st_rtsync>"max"            { return MAX; }
<st_rtsync>"enable"         { return ENABLE; }
<st_rtsync>"disable"        { return DISABLE; }
<st_rtsync>"action"         { return ACTION; }
<st_rtsync>"init"           { return INIT; }
<st_rtsync>"when"           { return WHEN; }  
<st_rtsync>"exception"      { return EXCEPTION; }
<st_rtsync>"->"             { return ARROW; }
<st_rtsync>"<->"            { return LINK; }
<st_rtsync>"sync"           { return SYNC; }

<st_rtsync>"["              { return BRACKET_OPEN; }
<st_rtsync>"]"              { return BRACKET_CLOSE; }
<st_rtsync>"("              { return PAREN_OPEN; }
<st_rtsync>")"              { return PAREN_CLOSE; }
<st_rtsync>"{" {
    scope_ctr++;
    return BRACE_OPEN;
}
<st_rtsync>"}" {
    scope_ctr--;
    if(!scope_ctr)
        BEGIN 0;
    return BRACE_CLOSE; 
}

<st_rtsync>"+"              { return PLUS; }
<st_rtsync>"-"              { return MINUS; }
<st_rtsync>"*"              { return TIMES; }
<st_rtsync>"/"              { return DIVIDE; }
<st_rtsync>"<"              { return LESS; }
<st_rtsync>"<="             { return LESS_EQ; }
<st_rtsync>">"              { return GREATER; }
<st_rtsync>">="             { return GREATER_EQ; }
<st_rtsync>"=="             { return EQUALS; }
<st_rtsync>"!="             { return NEQUALS; }
<st_rtsync>"="              { return ASSIGN; }
<st_rtsync>";"              { return SEMICOLON; }
<st_rtsync>","              { return COMMA; }
<st_rtsync>"."              { return DOT; }

<st_rtsync>\"[^"]*\" {
    yylval.txt = strdup(yytext);
    return STRING_LIT;
}

<st_rtsync>{LETTER}(_|{LETTER}|{DIGIT})* {
    yylval.txt = strdup(yytext);
    return ID;
}

<st_rtsync>("+"|"-")?{DIGIT}+"."{DIGIT}+ {
    yylval.txt = strdup(yytext);
    return FLOAT_LIT;
}

<st_rtsync>("+"|"-")?{DIGIT}+ {
    yylval.txt = strdup(yytext);
    return INT_LIT;
}

<st_rtsync>("+"|"-")?{DIGIT}+{LETTER}+ {
    yylval.txt = strdup(yytext);
    return ERROR;
}

<st_rtsync>.            { return ERROR; }

<INITIAL>. {
    yylval.txt = strdup(yytext);
    return PASS;
}

%%

