/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gsyncvisitor.h"
#include "codevisitor.h"

void gsyncvisitor::visit(int d, sync_decl_list_node* n)
{
    codevisitor v(out, graphout);
    v.cur_entity = "global";

    graphout << "<globalsync>\n";

    out << "class sync_global_t : public globalsync\n{\n";
    if(n->vars)
        n->vars->accept(1, v);
    out << "\npublic:\n";
    if(n->constraints)
        n->constraints->accept(1, *this);
    if(n->exception)
        n->exception->accept(1, v);
    out << "\n";
    out << indent(1) << "void constraint_check()\n"
        << indent(1) << "{\n"
        << indent(2) << "pthread_mutex_lock(&m_rhs_mutex);\n"
        << indent(2) << "notification_list::iterator it = rhs.begin();\n"
        << indent(2) << "while(it != rhs.end())\n"
        << indent(2) << "{\n"
        << indent(3) << "notification note = *it;\n"
        << indent(3) << "rhs.erase(it);\n"
        << indent(3) << "pthread_mutex_unlock(&m_rhs_mutex);\n"
        << rhs.str()
        << indent(3) << "pthread_mutex_lock(&m_rhs_mutex);\n"
        << indent(3) << "it = rhs.begin();\n"
        << indent(2) << "}\n"
        << indent(2) << "pthread_mutex_unlock(&m_rhs_mutex);\n"
        << indent(1) << "}\n\n";
    if(n->triggers)
        n->triggers->accept(1, v);
    out << "\n";
    n->init->accept(1, v);
    out << "};\n\n";

    graphout << "</globalsync>\n";

}

void gsyncvisitor::visit(int d, constraint_block_node* n) {
    out << indent(1) << "void c_notify_action(const actorref& a, "
        << "const method& m, const rttime_t& t)\n"
        << indent(1) << "{\n";
    if(n->head)
        n->head->accept(2, *this);
    out << indent(1) << "}\n\n";
}

void gsyncvisitor::visit(int d, constraint_list_node* n) {
    if(n->head) n->head->accept(d, *this);
    n->tail->accept(d, *this);
}


void gsyncvisitor::visit(int d, constraint_node* n)
{
    if(n->type == 0) // MIN
    {
        graphout << "min\n" << n->actor1 << "\n" << n->method1 << "\n"
                 << n->actor2 << "\n" << n->method2 << "\n";
        codevisitor gv(graphout);
        n->tail->accept(0, gv);
        graphout << "\n";

        // Notification is LHS of constraint
        out << indent(2) << "if(a == \"" << n->actor1
            << "\" and m == \"" << n->method1 << "\")\n"
            << indent(2) << "{\n"
            << indent(3) << "notification n = {a, m, t};\n"
            << indent(3) << "pthread_mutex_lock(&m_notify_mutex);\n"
            << indent(3) << "notification_list::iterator it = min_notes.find(n);\n"
            << indent(3) << "if(it == min_notes.end())\n"
            << indent(4) << "min_notes.insert(n);\n"
            << indent(3) << "else if(it->time < t)\n"
            << indent(3) << "{\n"
            << indent(4) << "min_notes.erase(it);\n"
            << indent(4) << "min_notes.insert(n);\n"
            << indent(3) << "}\n"
            << indent(3) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(2) << "}\n";

        // Notification is RHS of constraint
        out << indent(2) << "else if(a == \"" << n->actor2
            << "\" and m == \"" << n->method2 << "\")\n"
            << indent(2) << "{\n"
            << indent(3) << "notification n = { \"" << n->actor2 << "\", \"" << n->method2 << "\", rt_zero };\n"
            << indent(3) << "pthread_mutex_lock(&m_rhs_mutex);\n"
            << indent(3) << "notification_list::iterator it = rhs.find(n);\n"
            << indent(3) << "if(it == rhs.end())\n"
            << indent(4) << "rhs.insert(n);\n"
            << indent(3) << "else //if(it->time < t)\n"
            << indent(3) << "{\n"
            << indent(4) << "rhs.erase(it);\n"
            << indent(4) << "rhs.insert(n);\n"
            << indent(3) << "}\n"
            << indent(3) << "pthread_mutex_unlock(&m_rhs_mutex);\n"
            << indent(2) << "}\n";

        rhs << indent(3) << "if(note.a == \"" << n->actor2
            << "\" and note.m == \"" << n->method2 << "\")\n"
            << indent(3) << "{\n"
            << indent(4) << "notification n = { \"" << n->actor1 << "\", \"" << n->method1 << "\", rt_zero };\n"
            << indent(4) << "pthread_mutex_lock(&m_notify_mutex);\n"
            << indent(4) << "notification_list::iterator i = min_notes.find(n);\n"
            << indent(4) << "if(i != min_notes.end())\n"
            << indent(4) << "{\n"
            << indent(5) << "rttime_t dt = note.time - i->time;\n"
            << indent(5) << "if(!(dt >= rttime_t(0, ";
        codevisitor v(rhs, graphout);
        n->tail->accept(3, v);
        rhs << ")))\n"
            << indent(5) << "{\n"
            << indent(6) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(6) << "params p;\n"
            << indent(6) << "p << \"GLOBAL CONSTRAINT VIOLATION\";\n"
            << indent(6) << "message msg(i->a, i->m, note.a, note.m, p);\n"
            << indent(6) << "msg.timestamp.executed = note.time;\n"
            << indent(6) << "throw timing_exception(msg);\n"
            << indent(5) << "}\n"
            << indent(5) << "else\n"
            << indent(6) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(4) << "}\n"
            << indent(4) << "else\n"
            << indent(5) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(3) << "}\n";
    }
    else // MAX
    {
        graphout << "max\n" << n->actor1 << "\n" << n->method1 << "\n"
                 << n->actor2 << "\n" << n->method2 << "\n";
        codevisitor gv(graphout);
        n->tail->accept(0, gv);
        graphout << "\n";

        // Notification is LHS of constraint
        out << indent(2) << "if(a == \"" << n->actor1
            << "\" and m == \"" << n->method1 << "\")\n"
            << indent(2) << "{\n"
            << indent(3) << "notification n = {a, m, t};\n"
            << indent(3) << "pthread_mutex_lock(&m_notify_mutex);\n"
            << indent(3) << "notification_list::iterator it = max_notes.find(n);\n"
            << indent(3) << "if(it == max_notes.end())\n"
            << indent(4) << "max_notes.insert(n);\n"
            << indent(3) << "else if(it->time > t)\n"
            << indent(3) << "{\n"
            << indent(4) << "max_notes.erase(it);\n"
            << indent(4) << "max_notes.insert(n);\n"
            << indent(3) << "}\n"
            << indent(3) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(2) << "}\n";

        // Notification is RHS of constraint
        out << indent(2) << "else if(a == \"" << n->actor2
            << "\" and m == \"" << n->method2 << "\")\n"
            << indent(2) << "{\n"
            << indent(3) << "notification n = { \"" << n->actor2 << "\", \"" << n->method2 << "\", rt_zero };\n"
            << indent(3) << "pthread_mutex_lock(&m_rhs_mutex);\n"
            << indent(3) << "notification_list::iterator it = rhs.find(n);\n"
            << indent(3) << "if(it == rhs.end())\n"
            << indent(4) << "rhs.insert(n);\n"
            << indent(3) << "else //if(it->time > t)\n"
            << indent(3) << "{\n"
            << indent(4) << "rhs.erase(it);\n"
            << indent(4) << "rhs.insert(n);\n"
            << indent(3) << "}\n"
            << indent(3) << "pthread_mutex_unlock(&m_rhs_mutex);\n"
            << indent(2) << "}\n";

        rhs << indent(3) << "if(note.a == \"" << n->actor2
            << "\" and note.m == \"" << n->method2 << "\")\n"
            << indent(3) << "{\n"
            << indent(4) << "pthread_mutex_lock(&m_notify_mutex);\n"
            << indent(4) << "notification n = { \"" << n->actor1 << "\", \"" << n->method1 << "\", rt_zero };\n"
            << indent(4) << "notification_list::iterator i = max_notes.find(n);\n"
            << indent(4) << "if(i != max_notes.end())\n"
            << indent(4) << "{\n"
            << indent(5) << "rttime_t dt = note.time - i->time;\n"
            << indent(5) << "if(!(dt <= rttime_t(0, ";
        codevisitor v(rhs, graphout);
        n->tail->accept(3, v);
        rhs << ")))\n"
            << indent(5) << "{\n"
            << indent(6) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(6) << "params p;\n"
            << indent(6) << "p << \"GLOBAL CONSTRAINT VIOLATION\";\n"
            << indent(6) << "message msg(i->a, i->m, note.a, note.m, p);\n"
            << indent(6) << "msg.timestamp.executed = note.time;\n"
            << indent(6) << "throw timing_exception(msg);\n"
            << indent(5) << "}\n"
            << indent(5) << "else\n"
            << indent(6) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(4) << "}\n"
            << indent(4) << "else\n"
            << indent(5) << "pthread_mutex_unlock(&m_notify_mutex);\n"
            << indent(3) << "}\n";
    }
}
