/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "globalsync.h"
#include "postmaster.h"
#include "logger.h"

#include <string>

#include "timing.h"


globalsync::globalsync(const syncref& ref, postmaster& pm)
: PostMaster(pm), name(ref)
{
    pthread_mutex_init(&m_notify_mutex, 0);
    pthread_mutex_init(&m_rhs_mutex, 0);
}

// Polling functionality below

// To pass to pthread_create
void* start_polling_func_gs(globalsync* s)
{
    s->poll_master();
    return 0;
}

// Start the polling thread; each derived constructor shall call this once
// its members are initialized
void globalsync::start_polling()
{
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

    int result = pthread_create(&m_pollthread, &attr,
                                (void* (*)(void*)) start_polling_func_gs,
                                (void*) this);
#ifdef USE_SCHED
    sched_register(m_pollthread);
#endif

    if(result)
        throw std::string("Error: Synchronizer failed to create polling thread");
}

// Kill the polling thread; each derived destructor shall call this before
// destroying any of its members
void globalsync::end_polling()
{
    pthread_kill(m_pollthread, 9);
}

// Master polling function; do not touch
void globalsync::poll_master()
{
    rt_log << name << " starts its polling ..." << "\n";
    while(true)
    {
        try
        {
            constraint_check();
            poll_triggers();
        }
        catch(const timing_exception& te)
        {
            rt_log << "***Timing Exception: \n" << te << "\n\n";
            int val = exception_handler(te);
            if(val == TERMINATE)
                PostMaster.terminate_system();
            else if (val == KILLNODE)
                PostMaster.terminate_node();
        }
    }
} // end poll_master
