/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "messages.h"

message::serial_generator message::generator;
message null_msg = message::null_msg();

// Message end marker
const std::string params::END = "$";

// The following is included only to fix an inexplicable compilation error
// that occurred using GCC 3.3.4, when certain inner classes were somehow
// not exposed to the global namespace. The reasons are completely unknown,
// and the program compiled without a problem on GCC 3.4.1 without the
// mysterious hack.
message::serial_no stupid_gcc_334_error = message::serial_no(net_id(), 0);

message::serial_no message::serial_generator::next_serial()
{
    mut.lock();
    message::serial_no s = message::serial_no(nid, seq++);
    mut.unlock();
    return s;
}

// Specify net_id of the local system
void message::serial_generator::set_nid(net_id n)
{
    mut.lock();
    nid = n;
    mut.unlock();
}


// The stream operators are used to serialize messages for communication;
// the resultant strings are plain text but, while human readable, not
// necessarily human friendly -- they are designed to be easily parsed,
// not easily understood.

std::ostream& operator<<(std::ostream& os, const message& m)
{
    os << m.serial << " " << m.sender << " " << m.caller << " "
       << m.receiver << " " << m.callee << " " << m.sync << " "
       << m.timestamp.creation << " " << m.timestamp.dispatch << " "
       << m.timestamp.received << " " << m.timestamp.executed << " "
       << m.must_run_before << " " << m.must_run_after << " "
       << m.m_params;
    return os;
}

std::istream& operator>>(std::istream& is, message& m)
{
    is >> m.serial >> m.sender >> m.caller
       >> m.receiver >> m.callee >> m.sync
       >> m.timestamp.creation >> m.timestamp.dispatch
       >> m.timestamp.received >> m.timestamp.executed
       >> m.must_run_before >> m.must_run_after
       >> m.m_params;
    return is;
}

std::ostream& operator<<(std::ostream& os, const message::serial_no& s)
{
    os << s.nid << " " << s.seq;
    return os;
}

std::istream& operator>>(std::istream& is, message::serial_no& s)
{
    is >> s.nid >> s.seq;
    return is;
}

std::ostream& operator<<(std::ostream& os, const timing_exception& te)
{
    os << "timing exception at " << rt_gettime() << ":\n"
       << te.msg << "\n";
    return os;
}
