/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef MUTEX_H
#define MUTEX_H

#include <pthread.h>
#include <errno.h>

/*
 * Wrapper for mutexes.  Provides us with the functionality to inherit
 * from this class to create locked-structures.
 *
 * Member functions should be fairly self-explanatory.
 */
class mutex
{
    bool m_locked;
    pthread_mutex_t m_mutex;
public:
    mutex() {
        pthread_mutex_init(&m_mutex, 0);
    }
    virtual ~mutex() {
        if(m_locked)
            pthread_mutex_unlock(&m_mutex);
        pthread_mutex_destroy(&m_mutex);
    }
    void lock() {
        pthread_mutex_lock(&m_mutex);
        m_locked = true;
    }
    bool trylock() {
        int res = pthread_mutex_trylock(&m_mutex);
        if(res == EBUSY)
            return false;
        else
            return (m_locked = true);
    }

    // Note:  no protection on the unlock and officially POSIX states that
    //        unlocking an already-unlocked mutex can have unpredictable
    //        results.  This is necessary because we sometimes need to have
    //        this mutex unlocked by other calls like pthread_cond_wait and
    //        we therefore do not know the state.
    void unlock() {
        m_locked = false;
        pthread_mutex_unlock(&m_mutex);
    }
    pthread_mutex_t* get_pthread_mutex() {
        return &m_mutex;
    }
    bool locked() const {
        return m_locked;
    }
};

#endif // #define MUTEX_H
