/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef NET_ID_H
#define NET_ID_H

#include <boost/cstdint.hpp>
#include <iostream>

/*
 * This class holds an ip address and port.
 * Only ipv4 is supported at this time.
 */
class net_id
{
    boost::uint32_t ip_addr;
    boost::uint16_t dest_port;

public:
    net_id() : ip_addr(0), dest_port(0) { }

    net_id(boost::uint32_t ip, boost::uint16_t port)
    : ip_addr(ip), dest_port(port) { }

    // Comparison operators are needed so that we can use the net_id in
    // combination with serial numbers as unique message identifiers.

    bool operator==(const net_id& n) const
    {
        return (ip_addr == n.ip_addr) && (dest_port == n.dest_port);
    }

    bool operator!=(const net_id& n) const {
        return !(*this == n);
    }

    bool operator<(const net_id& n) const {
        return ip_addr < n.ip_addr ? true
               : ip_addr == n.ip_addr ? dest_port < n.dest_port : false;
    }

    // Return the ip address held within
    boost::uint32_t ip() { return ip_addr; }
    // Return the port held within
    boost::uint16_t port() { return dest_port; }

    // Stream operators
    friend std::ostream& operator<<(std::ostream& os, const net_id& id);
    friend std::istream& operator>>(std::istream& is, net_id& id);
};

#endif // #define NET_ID_H
