/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "params.h"

// Strings must be handled separately to encode parameter escape
// characters

params& params::operator<<(const std::string& s)
{
    // String format: $text here$
    m_stream << "$";
    std::stringstream ss(s);
    char c;
    while((c = ss.get()) && !ss.eof())
    {
        // '$' must be escaped!
        if(c == '$')
            m_stream << "\\$";
        else if(c == '\\')
            m_stream << "\\\\";
        else
            m_stream << c;
    }
    m_stream << "$ ";
    return *this;
}

params& params::operator>>(std::string& s)
{
    std::stringstream ss;
    while(m_stream.get() != '$');
    char lastchar = '\0';
    while(true)
    {
        char c = m_stream.get();
        // Look for escaped escape char ...
        if(c == '\\')
        {
            if(lastchar == '\\')
                ss << '\\';
        }
        // If we find '$', was it escaped with '\' or is it the end
        // marker?
        else if(c == '$')
        {
            if(lastchar == '\\')
                ss << '$';
            else
                break;
        }
        else
        {
            if(lastchar == '\\')
                ss << lastchar << c;
            else
                ss << c;
        }
        lastchar = c;
    }
    s = ss.str();
    return *this;
}


std::ostream& operator<<(std::ostream& os, const params& p)
{
    os << p.m_stream.str();
    return os;
}

std::istream& operator>>(std::istream& is, params& p)
{
    if(is.peek() == ' ') is.ignore(1);
    char c;
    while((is.get(c))) p << c;
    is.clear();
    return is;
}
