/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef PQUEUE_H
#define PQUEUE_H

#include <algorithm>
#include <vector>

// Templated priority queue class...general use.
// Provides standard container functions but is not guaranteed to be
// complete.
template <typename T> class pqueue
{
    typedef std::vector<T> container;
    container m_data;
    int m_size;
public:
    pqueue() : m_size(0) { }

    pqueue(const container& c) : m_data(c), m_size((int) c.size())
    {
        std::make_heap(m_data.begin(), m_data.end());
    }

    pqueue(typename container::iterator first,
           typename container::iterator last)
    {
        m_data = container(first, last);
        m_size = (int) m_data.size();
        std::make_heap(m_data.begin(), m_data.end());
    }

    void enqueue(const T& data)
    {
        m_data.push_back(data);
        m_size++;
        std::push_heap(m_data.begin(), m_data.begin()+m_size);
    }

    T dequeue()
    {
        T ret = *m_data.begin();
        std::pop_heap(m_data.begin(), m_data.begin()+m_size--);
        m_data.pop_back();
        return ret;
    }

    void resort()
    {
        std::make_heap(m_data.begin(), m_data.begin()+m_size);
    }

    T& peek()
    {
        return m_data.front();
    }

    int size() const { return m_size; }

    typedef typename container::iterator iterator;
    iterator begin() { return m_data.begin(); }
    iterator end()   { return m_data.begin()+m_size; }
};


#endif // #define PQUEUE_H
