/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef SCHEDULABLE_H
#define SCHEDULABLE_H

#include "scheduler.h"
#include "logger.h"

#include <pthread.h>
#include <signal.h>
#include <iostream>

class rttime_t;

#include <unistd.h>

// Used for synchronizers.  Every synchronizer must inherit from this struct.
struct rt_sched_sync
{
    pthread_cond_t rt_sched_token;  // condition variable for thread to wait on
    static pthread_mutex_t cond_table_lock;
    bool rt_sched_bool; // do we have the token?

    rt_sched_sync() : rt_sched_bool(false)
    {
        pthread_cond_init(&rt_sched_token,0);
    }

    virtual ~rt_sched_sync() { }

    // Registers thread with the scheduler, then signals and locks it.
    void sched_register(pthread_t pid)
    {
        pthread_mutex_lock(&cond_table_lock);
        cond_table_sync[pid] = this;
        pthread_mutex_unlock(&cond_table_lock);
        signal(SIGUSR2, rt_sighandler_sync);
        pthread_kill(pid, SIGUSR2); // thread, block thyself
    }
};

// Every actor must inherit from this struct.
struct rt_sched_actor
{
    pthread_cond_t rt_sched_token; // condition variable for thread to wait on
    static pthread_mutex_t cond_table_lock;
    bool rt_sched_bool; // do we have the token?
    bool rt_sched_been_sched;  // have we reset the priority this cycle?

    // Override in inheriting classes: Return the deadline
    virtual rttime_t getpriority() const = 0;

    rt_sched_actor()
    : rt_sched_bool(false), rt_sched_been_sched(false)
    {
        pthread_cond_init(&rt_sched_token,0);
    }

    virtual ~rt_sched_actor() { }

    // Registers thread with the scheduler, then signals and locks it.
    void sched_register(pthread_t pid)
    {
        pthread_mutex_lock(&cond_table_lock);
        cond_table_actor[pid] = this;
        pthread_mutex_unlock(&cond_table_lock);
        signal(SIGUSR1, rt_sighandler_actor);
        pthread_kill(pid, SIGUSR1); // thread, block thyself
    }
};

#endif // #define SCHEDULABLE_H
