/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef SCHEDULER_H
#define SCHEDULER_H

#include <map>
#include <pthread.h>


class rt_sched_actor;
class rt_sched_sync;

// map keyed by pthread_t to look up rt_sched_sync* which because synchronizers
// inherit from them, contain the condition variables that the sync will lock
// on in the signal handler
extern std::map<pthread_t, rt_sched_sync*> cond_table_sync;
// same as above except for actors
extern std::map<pthread_t, rt_sched_actor*> cond_table_actor;

// Function to start the scheduler
pthread_t sched_begin();

/* Signal handler to catch signals sent to actor schedulable entities.
 * Pre: the actor thread must be running before being sent a signal.  If
 *      this is not the case, we will call this function from within this
 *      function, which may have unpredictable results.
 * The thread will sit here waiting on the pthread_cond_signal to leave
 * and continue execution
 * Post: the actor thread will continue running at the location in its
 *       code where it left to handle the signal.
 */
void rt_sighandler_actor(int sig);

/* Signal handler to catch signals sent to synchronizer schedulable entities.
 * Pre: the sync thread must be running before being sent a signal.  If
 *      this is not the case, we will call this function from within this
 *      function, which may have unpredictable results.
 * The thread will sit here waiting on the pthread_cond_signal to leave
 * and continue execution
 * Post: the sync thread will continue running at the location in its
 *       code where it left to handle the signal.
 */
void rt_sighandler_sync(int sig);

#endif // #define SCHEDULER_H
