/*
 * RTSync: real-time compiler and runtime environment
 * Copyright (C) 2004  Stefan D. Bruda, Petter Haggholm, and Scott Stoddard
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named LICENSE; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <iomanip>
#include <sstream>
#include <limits>

#include "timing.h"

const rttime_t rt_inf(std::numeric_limits<time_t>::max(), 0);
const rttime_t rt_zero(0, 0);
const rttime_t rt_startup = rt_gettime();

std::ostream& operator<<(std::ostream& os, const rttime_t& t)
{
    using namespace std;
    os << t.tv_sec << ":" << setw(3) << setfill('0') << t.tv_usec/1000
       << "." << setw(3) << setfill('0') << t.tv_usec%1000;
    return os;
}

std::istream& operator>>(std::istream& is, rttime_t& t)
{
    std::stringstream ss;

    is.get(*ss.rdbuf(), ':');
    is.ignore(1, ':');
    ss >> t.tv_sec;
    ss.str("");

    unsigned long ms, us;

    is.get(*ss.rdbuf(), '.');
    is.ignore(1, '.');
    ss >> ms;
    ss.str("");

    is >> us;

    t.tv_usec = ms*1000 + us;

    return is;
}
